#ifndef PARSE_NODES_H
#define PARSE_NODES_H

#include <stdlib.h>
#include <stdio.h>
#include <iostream.h>
#include <fstream.h>

class GeneralNode {
  public:
    virtual void print(ostream& os = cout) {}
};

#define YYSTYPE GeneralNode*

class StmtsNode : public GeneralNode {
  private:
    StmtsNode() {}
  public:
    GeneralNode *stmt;
    GeneralNode *next;
    StmtsNode(GeneralNode *e, GeneralNode *nxt);
    void print(ostream& os = cout);
};

class BinaryNode : public GeneralNode {
  private:
    BinaryNode() {}
  public:
    int theop;
    GeneralNode *leftarg;
    GeneralNode *rightarg;
    BinaryNode(int op, GeneralNode *arg1, GeneralNode *arg2);
    void print(ostream& os = cout);
};

class UnaryNode : public GeneralNode {
  private:
    UnaryNode() {}
  public:
    int theop;
    GeneralNode *arg;
    UnaryNode(int op, GeneralNode *thearg);
    void print(ostream& os = cout);
};

class IntNode : public GeneralNode {
  private:
    IntNode() {}
  public:
    int ivalue;
    IntNode(int ival);
    void print(ostream& os = cout);
};

extern GeneralNode *parse_root;

GeneralNode *do_parse(const char* filename);

extern char *yytext;
extern int yylex();
extern FILE *yyin;
extern void yyerror(const char *s);
extern YYSTYPE yylval;

extern int char_num;
extern int line_num;
extern int s_char_num;
extern int s_line_num;

#endif
